'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _flatMap = require('lodash/flatMap');

var _flatMap2 = _interopRequireDefault(_flatMap);

var _uniqueId = require('lodash/uniqueId');

var _uniqueId2 = _interopRequireDefault(_uniqueId);

var _classnames2 = require('classnames');

var _classnames3 = _interopRequireDefault(_classnames2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } // npm dependencies
//


/**
 * Build a list of <option> elements from a list of options
 * @param  {Array} opts The options for which to build the list of elements
 * @return {Array}      An array of <option> elements
 */
var optionsList = function optionsList(opts) {
    return opts.map(function (opt) {
        return _react2.default.createElement(
            'option',
            { key: (0, _uniqueId2.default)('form-dropdown-option-'), value: opt.value || opt },
            opt.name || opt
        );
    });
};

/**
 * Flatten the list of options
 * @param  {Array|Object} options The list of options
 * @return {Array}                The flattened list of options
 */
var flattenOptionsList = function flattenOptionsList(options) {
    return (0, _isArray2.default)(options) ? options : (0, _flatMap2.default)(options, function (opt) {
        return opt;
    });
};

/**
 * The Dropdown component
 */

var Dropdown = function (_React$Component) {
    _inherits(Dropdown, _React$Component);

    /**
     * Construct the component instance
     * @param  {Object} props The component props
     */
    function Dropdown(props) {
        _classCallCheck(this, Dropdown);

        // generate a unique ID for this instance
        //
        var _this = _possibleConstructorReturn(this, (Dropdown.__proto__ || Object.getPrototypeOf(Dropdown)).call(this, props));

        _this.id = props.id || (0, _uniqueId2.default)('form-dropdown-');

        // do we have a value? This is an interesting situation... the user has
        // not provided a value, so the value *ought* to be the first item in
        // the Dropdown.
        //
        // I attempted to fix this by "pushing" the updated value to the state
        // via the `onChange` handler, but that's a no-no in React.
        //
        // Possible solutions:
        // 1. require the `value` parameter - but what happens if it's not
        //      valid?
        // 2. push the new value after the component has mounted - but this
        //      requires implementing state on this component
        //
        // I'm going to try number 2.

        // build the list of options for the component
        //
        _this.optionsList = flattenOptionsList(props.options || []);

        // get the first value from the list of options (if it exists)
        //
        _this.firstValue = _this.optionsList[0] ? _this.optionsList[0].value : '';

        // validate & save the value
        //
        var value = props.value && _this.isValid(props.value) ? props.value : _this.firstValue;

        // initialize the component state
        //
        _this.state = { value: value };

        // bind `this` to the event handlers
        //
        _this.isValid = _this.isValid.bind(_this);
        _this.onChange = _this.onChange.bind(_this);
        return _this;
    }

    /**
     * Handle the "component just mounted" event
     */


    _createClass(Dropdown, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            // does the value in the props equal the value in the state? if not, and
            // if we have an `onChange` handler, then call the `onChange` handler
            //
            if (this.props.value !== this.state.value && this.props.onChange) {
                this.props.onChange(this.state.value);
            }
        }

        /**
         * Handle the `onChange` event for the select element
         * @param  {Object} event The event object
         */

    }, {
        key: 'onChange',
        value: function onChange(event) {
            var _this2 = this;

            // figure out which value to use
            //
            var newValue = this.isValid(event.target.value) ? event.target.value : this.firstValue;

            // update the state
            //
            this.setState({ value: newValue }, function () {
                // ...then call the `onChange` handler, if it exists
                //
                if (_this2.props.onChange) {
                    _this2.props.onChange(_this2.state.value);
                }
            });
        }

        /**
         * Determine if a value is valid (ie. in the list of options)
         * @param  {String}  value The value to check
         * @return {Boolean}       `true` if the value is valid (ie. found in the
         *                         array of options); `false` otherwise
         */

    }, {
        key: 'isValid',
        value: function isValid(value) {
            return this.optionsList.findIndex(function (opt) {
                return '' + opt.value === '' + value;
            }) !== -1;
        }

        /**
         * Render the component
         * @return {React.Element} The React element describing the component
         */

    }, {
        key: 'render',
        value: function render() {
            var _this3 = this;

            // generate the label for the component
            //
            var label = this.props.label ? _react2.default.createElement(
                'label',
                {
                    htmlFor: this.id,
                    className: (0, _classnames3.default)('control-label', _defineProperty({}, 'col-xs-' + this.context.labelColumns, this.context.labelColumns))
                },
                this.props.label
            ) : null;

            // build the options
            //
            var options = null;

            // is the list of options an array or an object?
            //
            if ((0, _isArray2.default)(this.props.options)) {
                // it's an array - simply map it to a list of <option> elements
                //
                options = optionsList(this.props.options);
            } else {
                // it's an object - build a list of options, separated into
                // optgroups by key
                //
                options = [];

                Object.keys(this.props.options).forEach(function (key) {
                    options.push(_react2.default.createElement(
                        'optgroup',
                        { key: (0, _uniqueId2.default)('form-dropdown-optgroup-'), label: key },
                        optionsList(_this3.props.options[key])
                    ));
                });
            }

            // generate the select element for the component
            //
            var select = _react2.default.createElement(
                'select',
                {
                    id: this.id,
                    name: this.props.name,
                    disabled: !!this.props.disabled,
                    className: 'form-control',
                    value: this.state.value,
                    onChange: this.onChange
                },
                options
            );

            // do we have columns for this component? if so, wrap the select element
            // in a div to implement those columns
            //
            if (this.context.labelColumns) {
                select = _react2.default.createElement(
                    'div',
                    {
                        className: (0, _classnames3.default)('form-dropdown-columns', 'col-xs-' + (12 - this.context.labelColumns))
                    },
                    select
                );
            }

            // return the component
            //
            return label ? _react2.default.createElement(
                'div',
                { className: 'form-group' },
                label,
                select
            ) : select;
        }
    }]);

    return Dropdown;
}(_react2.default.Component);

/**
 * The property types for the component
 * @type {Object}
 */


Dropdown.propTypes = {
    /** A flag to indicated whether this component is disabled */
    disabled: _react2.default.PropTypes.bool,
    /** The ID for the component */
    id: _react2.default.PropTypes.string,
    /** The label for the checkbox group */
    label: _react2.default.PropTypes.string,
    /** The name for the select element */
    name: _react2.default.PropTypes.string,
    /** The selected value */
    value: _react2.default.PropTypes.oneOfType([_react2.default.PropTypes.string, _react2.default.PropTypes.number]),
    /**
     * The options for the dropdown, in one of three possible forms:
     * + An array of strings - the string is used as both the value & name for
     *   the option
     * + An array of objects of the form  { value: 'x', name: 'y' }
     * + An object where the key is the option group name and each value is
     *   either an array of strings or an array of objects as described above
     */
    options: _react2.default.PropTypes.oneOfType([_react2.default.PropTypes.array, _react2.default.PropTypes.object]).isRequired,
    /** The 'onChange' handler for the component */
    onChange: _react2.default.PropTypes.func
};

/**
 * The context types for the component
 * @type {Object}
 */
Dropdown.contextTypes = {
    labelColumns: _react2.default.PropTypes.number
};

// export the component
//
exports.default = Dropdown;